<?php
/**
 * Tarjei's ldapclasses.
 *
 * Made because I want to do things my way :/
 * ( or realy the perl net::ldap way)
 *
 * @author  Tarjei Huse
 * @version $Id$
 * @package Net_LDAP
 */

/**
 * Include required files
 */
require_once 'PEAR.php';
require_once 'LDAP/Entry.php';
require_once 'LDAP/Search.php';

/**
 * Main class
 *
 * @package Net_LDAP
 */
class Net_Ldap extends PEAR
{
    /**
     * Class configuration array
     * dn = the DN to bind as.
     * host = the ldap host to connect to
     * password = no explanation needed
     * base = ldap base
     * port = the server port
     * tls - is set - the ldap_start_tls() is run after connecting.
     * version = ldap version (defaults to v 3)
     *
     * @var array
     */
     var $_config = array ('dn',
         'host' => 'localhost',
         'password',
         'tls' => false,
         'base' => '',
         'port' => 389,
         'version' => 3
        );

    /**
     * The ldap resourcelink.
     * You should not need to touch this.
     *
     * @var object
     */
     var $_link;

    /**
     * Net ldap, Sets the configarray and binds if the config array contains the server prameter
     *
     * @access protected
     * @param array $config - se description in class vars
     * @return void
     */
    function Net_Ldap($_config = array(), $bind = false)
    {
        foreach ($_config as $k => $v) {
            $_{$k} = $v;
        }
    }

    /**
     * connect - create the initial ldap-object
     *
     *
     * function &connect($_config = array())
     *
     * Static function that returns either an error object or the new Net_ldap object.
     *
     * @access public
     * @param  $_config array containgin the needed ldap configuration parameters. Defaults:
     *                  'dn',
     *                  'host' => 'localhost',
     *                  'password',
     *                  'tls' => false,
     *                  'base' => '',
     *                  'port'=>389,
     *                  'version'=> 3
     * @return reference object , either a PEAR error or a Net_ldap object.
     */
    function &connect($_config = array())
    {
        @$obj = & new Net_Ldap;
        $err  = $obj -> bind($_config);

        if (Net_Ldap :: isError($err)) {
            return $err;
        }

        return $obj;
    }

    /**
     * Bind to the ldap-server
     *
     * The function may be used if you do not create the object using Net_ldap::connect.
     *
     * @access public
     * @param  array $ config see connect for the structure and defaults.
     * @return mixed true if the bind went ok, Net_ldap_error if not.
     */
    function bind($config = array())
    {
        foreach ($config as $k => $v) {
            $this -> _config[$k] = $v;
        }

        if ($this -> _config['host']) {
             $conn = ldap_connect($this -> _config['host'], $this -> _config['port']);
        } else {
             return $this -> raiseError("Unknown host {$this->_config['host']}");
        }

        if (!$conn) {
             return $this -> raiseError("LDAP_CONNECT_FAILED");
        }
        // You must set the version and start tls BEFORE binding!
        // (quite logical when you think of it...
        if ($this -> _config['version'] == 3 && !@ldap_set_option($conn, LDAP_OPT_PROTOCOL_VERSION, 3)) {
            return $this -> raiseError("Could not set ldap v3: " . ldap_error($conn));
        }

        if ($this -> _config['tls'] ) {
            if (!@ldap_start_tls($conn)) {
                return $this -> raiseError("TLS not started. Error:" . ldap_error($conn));
            }
        }
                   
        
        if (isset($this -> _config['dn']) && isset($this -> _config['password'])) {
             $bind = @ldap_bind($conn, $this -> _config['dn'], $this -> _config['password']);
        } else {
             $bind = @ldap_bind($conn);
        }

        if (!$bind) {
             return $this -> raiseError("Bind failed " . @ldap_error($conn)  );
        }

        $this -> _link = $conn;

        return true;
    }

    /**
     * start_tls() - start an encrypted session
     *
     * Starts an encrypted session
     *
     * @access public
     * @return - ldap_error if error else true
     */
    function start_tls()
    {
        if (!@ldap_set_option($this -> _link, LDAP_OPT_PROTOCOL_VERSION, 3)) {
             return $this -> raiseError("Could not set ldap v3" . ldap_error($this -> _link));
        }
        return true;
    }

    /**
     * done - close ldap-connection.
     *
     * Close the ldapconnection. Use this when the session is over.
     *
     * @access private
     * @return void
     */
    function done()
    {
        $this -> _Net_Ldap();
    }

    /**
     * Destructor
     *
     * @access private
     */
    function _Net_Ldap()
    {
        @ldap_close($this -> _link);
    }

    /**
     * add - add a new entryobject to a directory.
     *
     * Use add to add a new Net_ldap_entry object to a directory.
     *
     * @param  $ - $entry a Net_ldap_entry object.
     * @return mixed Net_ldap_error if feilure true if sucess.
     */
    function add($entry)
    {
        if (@ldap_add($this -> _link, $entry -> dn(), $entry -> get_attrs())) {
             return 'true';
        } else {
             return $this -> raiseError("Could not add entry " . $entry -> dn() . " " . ldap_error());
        }
    }

    /**
     * delete - Delete an object,
     *
     * Delete an object, the object may either be a string representing the dn or a ldap_entry object.
     *
     * @access public
     * @params mixed $dn - string representing the dn or a Net_ldap_entry object
     */
    function delete($dn)
    {
        if (is_object($dn) && get_class($dn) == 'ldap_entry') {
             $dn = $dn -> dn();
        } else {
            if (!is_string($dn)) {
                 return $this -> raiseError("$dn not a string nor an entryobject!");
            }
        }

        if (!@ldap_delete($this -> _link, $dn)) {
             return $this -> raiseError("Could not delete entry " . $dn . " \n" . ldap_error());
        }

        return true;
    }

    /**
     * modify - modify an ldapentry
     *
     *
     * This is taken from the perlpod of net::ldap, and explains things quite nicely.
     * modify ( DN, OPTIONS )
     * Modify the contents of DN on the server. DN May be a
     * string or a Net::LDAP::Entry object.
     *
     * dn  This option is here for compatibility only, and
     * may be removed in future.  Previous releases did
     * not take the DN argument which replaces this
     * option.
     *
     * add The add option should be a reference to a HASH.
     * The values of the HASH are the attributes to add,
     * and the values may be a string or a reference to a
     * list of values.
     *
     * delete
     * A reference to an ARRAY of attributes to delete.
     * TODO: This does not support deleting one or two values yet - use
     * replace.
     *
     * replace
     * The <replace> option takes a argument in the same
     * form as add, but will cause any existing
     * attributes with the same name to be replaced. If
     * the value for any attribute in the rray is a ref
     * erence to an empty string the all instances of the
     * attribute will be deleted.
     *
     * changes
     * This is an alternative to add, delete and replace
     * where the whole operation can be given in a single
     * argument. The argument should be a array
     *
     * Values in the ARRAY are used in pairs, the first
     * is the operation add, delete or replace and the
     * second is a reference to an ARRAY of attribute
     * values.
     *
     * The attribute value list is also used in pairs.
     * The first value in each pair is the attribute name
     * and the second is a reference to a list of values.
     *
     * Example
     * $ldap->modify ( $dn, array (changes=>array(
     * 'delete' => array('faxNumber'=>''),
     * 'add'=>array('sn'=>'Barr'),
     * 'replace' => array(email=>'tarjei@nu.no)
     * )
     * )
     * );
     *
     * @access public
     * @param string $dn representing a DN, array $params containing the changes.
     * @return mixed : Net_ldap_error if failure and true if success.
     */
    function modify($dn , $params = array())
    {
        if (is_object($dn)) {
             $dn = $dn -> dn();
        }

         // since $params['dn'] is not used in net::ldap now:
        if (isset($params['dn'])) {
             return $this -> raiseError("This feature will not be implemented!");
        }

        if (isset($params['changes'])) {

             if (isset($params['changes']['add']) &&
                     !@ldap_modify($this -> _link, $dn, $params['changes']['add'])) {

                 return $this -> raiseError("Add didn't work" . ldap_error($this -> _link));
             }

             if (isset($params['changes']['replace']) &&
                     !@ldap_modify($this -> _link, $dn, $params['changes']['replace'])) {

                 return $this -> raiseError("replace change didn't work: " . ldap_error($this -> _link));
             }

             if (isset($params['changes']['delete']) &&
                     !@ldap_mod_del($this -> _link, $dn, $params['changes']['delete'])) {

                 return $this -> raiseError("delete did not work" . ldap_error($this -> _link));
             }
        }

        if (isset($params['add']) && !@ldap_add($this -> _link, $dn, $params['add'])) {
            return $this -> raiseError(ldap_error($this -> _link));
        }

        if (isset($params['replace']) && !@ldap_modify($this -> _link, $dn, $params['replace'])) {
            return $this -> raiseError(ldap_error($this -> _link));
        }

        if (isset($params['delete'])) {
             // since you delete an attribute by making it empty:
            foreach ($params['delete'] as $k) {
                $params['delete'][$k] = '';
            }

            if (!@ldap_modify($this -> _link, $dn, $params['delete'])) {
                 return $this -> raiseError(ldap_error($this -> _link));
            }
        }
        // everything went fine :)
        return true;
    }

    /**
     * search() - run a ldap query
     *
     *   Search is used to query the ldap-database. Seardh will either return an Net_ldap_search object
     *   or an Net_ldap_error object.
     *
     * @access public
     * @param  $base - ldap searchbase, $filter - ldap filter, both may be omitted returning a search after uid=* for the whole server.
     * @return object Net_ldap_search or Net_ldap_error
     */
    function search($base = null, $filter = null)
    {
        if ($base == null) {
            $base = $this -> _config['base'];
        }
        if ($filter == null) {
            $filter = '(uid=*)';
        }

        $search = @ldap_search($this-> _link, $base, $filter);
        
        if (ldap_errno($this->_link)) { // ldap_errno : 0 = sucess, the rest I'll put in here some day...
            return $this->raiseError("Net_LDAP::search, function ldap_search got error " . ldap_error($this->_link) . " Params: base: $base, filter: $filter");
        } else {
            @$obj = & new Net_Ldap_Search(& $search, $this -> _link);
            // $obj->_set_search(&$search,$this->_link);
           return $obj;
        }
        /* OLDcode left here for me to remember!
        } else {
            return $this -> raiseError("LDAP: No objects found");
        }
        */

    }
}

// Class ldap_search implements my own search_class

/**
 * LDAP_Error implements a class for reporting portable ldap error
 * messages.
 *
 * @package Net_LDAP
 * @author  Stig Bakken <ssb@fast.no>
 */
class Net_Ldap_Error extends PEAR_Error
{
    /**
     * LDAP_Error constructor.
     *
     * @param mixed $code DB error code, or string with error message.
     * @param integer $mode what "error mode" to operate in
     * @param integer $level what error level to use for $mode & PEAR_ERROR_TRIGGER
     * @param mixed $debuginfo additional debug info, such as the last query
     * @access public
     * @see PEAR_Error
     */
    function LDAP_Error($code = DB_ERROR, $mode = PEAR_ERROR_RETURN,
         $level = E_USER_NOTICE, $debuginfo = null)
    {
        if (is_int($code)) {
            $this -> PEAR_Error('LDAP Error: ' . LDAP :: errorMessage($code), $code, $mode, $level, $debuginfo);
        } else {
            $this -> PEAR_Error("LDAP Error: $code", LDAP_ERROR, $mode, $level, $debuginfo);
        }
    }
}
?>
