<?php
class Net_Ldap_Entry extends PEAR
{

    /**
     * Array of the attributes the user has
     * */
    var $_attrs = array(); // the attribute array
    var $_dn = ''; // The dn
    var $_link = null; // ldap resourcelink.
    
    var $_olddn = ''; // The users old DN if the dn has been changed


    // updatechecks
    var $updateCheck = array('newdn'    => false,
                             'modify'   => false,
                             'newEntry' => true
                             ); // since the entry is not changed before the update();

    /** Constructor
     * @param - link - ldap_resource_link, dn = string entry dn, attributes - array entry attributes array.
     * @return - none
     **/
    function Net_Ldap_Entry ($link = null, $dn = null, $attributes = array())
    {

        if (!is_null($link)) {
            $this->_link = $link;
        }
        if (!is_null($dn)) {
            $this->_set_dn($dn);
        }
        if (count ($attributes)>0 ) {
            $this->_set_attributes($attributes);
        }


    }
    /** Set the reasourcelink to the ldapserver.
     *
     * */
    function _set_link(&$link) 
    {
        $this->_link = $link;
    }
    /** _set_dn - set the entrys DN 
     * */
    function _set_dn ($dn)
    {
        $this->_dn = $dn;
    }

    /** _set_attributes - sets the internal array of the entrys attributes.
     * 
     * */
    function _set_attributes ($attributes= array())
    {
        $this->_attrs = $attributes;
        // this is the sign that the entry exists in the first place: 
        $this->updateCheck['newEntry'] = false;
    }

   /** clean_entry - removes [count] entries from the array.
    * 
   * remove all the count elements in the array:
   * Used before ldap_modify, ldap_add
   * 
   * @params - none
   * */
    function _clean_entry()
    {
        $attributes = array();
        
        for ($i=0; $i < $this->_attrs['count'] ; $i++) {
        
            $attr = $this->_attrs[$i];
        
            if ($this->_attrs[$attr]['count'] == 1) {
                $attributes[$this->_attrs[$i]] = $this->_attrs[$attr][0];
            } else {
                $attributes[$attr] = $this->_attrs[$attr];
                unset ($attributes[ $attr ]['count']);
            }
        }
        
        return $attributes;

    }

   /** attributes -  returns an assosiative array of all the attributes in the array
    *
    * attributes -  returns an assosiative array of all the attributes in the array
    * on the form array ('attributename'=>'singelvalue' , 'attribute'=>array('multiple','values'))
    * @param none
    * @return $array of attributes and values.
   */

    function attributes ()
    {
        return $this->_clean_entry();
    }

   /** add -  Add one or more attribute to the entry
    *
    * The values given will be added to the values which already exist for the given attributes.
    * usage:
    * $entry->add ( array('sn'=>'huse',objectclass=>array(top,posixAccount)))
    * @param $array of attributes example: array('sn'=>'huse',objectclass=>array(top,posixAccount))
    * @return Net_Ldap_Error if error, else true.
    */

    function add ($attr = array())
    {

        if (!isset($this->_attrs['count']) ) {
            $this->_attrs['count'] = 0;
        }

        foreach ($attr as $k => $v ) {
            if ($this->exists($k)) {
                array_push($this->_attrs[$k],$v);
                $this->_attrs[$k]['count']++;

            } else {
                $this->_attrs[$k][0] = $v;
                $this->_attrs[$k]['count'] = 1;
                $this->_attrs[$this->_attrs['count']] = $k;
                $this->_attrs['count']++;
            }
        }
        return true;
    }

   /** dn - Set or get the DN for the object
    *
    * If a new dn is supplied, this will move the object when running $obj->update();
    * @param - string DN
    * @return none
    */
    function dn ($newdn="")
    {
        if ($newdn == "") {
            return $this->_dn;
        }
      
        $this->_olddn = $this->_dn;
        $this->_dn = $newdn;
        $updateCheck['newdn'] = true;
    }
   
   /** exists - check if a certain attribute exists in the directory
    *
    * Checks if the entry contains a certain attribute.
    * @params string attribute name.
    * @return boolean
   */
    function exists ($attr)
    {
        if (array_key_exists($attr,$this->_attrs)) {
            return true;
        }
    
        return false;
    }

   /** get_value get the values for a attribute
    * returns either an array or a string
    * $attr is a string with the attribute name.
    *
    *
    * @param $attr string attribute name
    *        $options assoiative array, possible values:
    *           alloptions - returns an array with the values + a countfield.
    *                       i.e.: array (count=>1, 'sn'=>'huse');
    *           single - returns the, first value in the array as a string.
    */
    function get_value($attr =  '',$options = '')
    {

        if (array_key_exists($attr,$this->_attrs)) {

            if ($options == 'single') {
                return $this->_attrs[$attr][0];
            }

            $value = $this->_attrs[$attr];
            
            if (!$options == 'alloptions' ) {
                unset ( $value['count'] );
            }

            return  $value;
            
        } else {
            return '';
        }
    }

   /** replace - replace a certain attributes value
    *
    * replace - replace a certain attributes value
    * example:
    * $entry->replace(array('uid'=>array('tarjei')));
    * @param array attributes to be replaced
    * @return error if failure, true if sucess.
    */
    function replace ($attrs = array() )
    {

        foreach ($attrs as $k => $v) {
           
            if ($this->exists($k)) {

                $this->_attrs[$k] = $v;
                
                if (is_array($v)) {
                    $this->_attrs[$k]['count'] = count($v);
                } else {
                    $this->_attrs[$k]['count'] = 1;
                }

            } else {
                return $this->raiseError("Attribute does not exist");
            }
        }
        return true;
    }

   /** delete -  delete attributes
    *
    * Use this function to delete certian attributes from an object.
    *
    * @param - array of attributes to be deleted
    * @return mixed Net_Ldap_Error if failure, true if success.
    * */
    function delete($attrs = array())
    {

        foreach ($attrs as $k => $v) {
            
            if ($this->exists ($k)) {
                // if v is a null, then remove the whole attribute, else only the value.
                if ($v = '') {
                    unset($this->_attrs[$k]);
                
                // else we remove only the correct value.
                } else {
                
                    for ($i = 0;$i< $this->_attrs[$k]['count'];$i++) {
                        if ($this->_attrs[$k][$i] == $v ) {
                            unset ($this->_attrs[$k][$i]);
                            continue;
                        }
                    }
                    
                }
                
            } else {
                $this->raiseError("You tried to delete a nonexisting attribute!");
            }


        }
        
        return true;
    }

   /** update -  update the ldapobject
    *
    * After modifying an object, you must run update() to
    * make the updates on the ldap server. Before that, they only exists in the object.
    *
    * @param - none
    * @return Net_Ldap_Error if failure to update the object.
    *
    * */
    function update ($ldapObject = null)
    {

        if ($ldapObject == null && $this->_link == null ) {
            $this->raiseError("No link to database");
        }

        if ($ldapObject != null) {
            $this->_link = $ldapObject->_link;
        }

        if ($updateCheck['newdn']) {
          //    ldap_rename ( resource link_identifier, string dn, string newrdn, string newparent, bool deleteoldrdn)
            $newparent = ldap_explode_dn($this->_dn);
            // remove the first part
            array_pop($newparent);
            if (!@$ldap_rename( $this->_link,$this->_olddn,$this->_dn,$newparent,true) ){
                 return $this->raiseError("DN not renamed: " . ldap_error($this->_link));
            }
        }

        if ($this->updateCheck['newEntry']) {
           //print "<br>"; print_r($this->_clean_entry());

            if (!@ldap_add($this->_link, $this->dn(), $this->_clean_entry())) {
                  return $this->raiseError("Entry not added!" . ldap_error($this->_link));
            } else {
                return "";
            }
            
        } else {
            
            if (!@ldap_modify($this->_link, $this->dn(), $this->_clean_entry())) {
                return $this->raiseError("Entry not modified: " . ldap_error($this->_link));
            }
            
            return "";
        }

    }
}

?>
